@@ -466,11 +466,6 @@ class Campaign:
         """
         # Get reference epoch:
         self.ref_delta_t_dt = self.get_epoch_of_first_observation(active_obs_only_for_ref_epoch)
-        # if set_epoch_of_first_obs_as_reference:
-        #     # Get epoch of first observation:
-        #     self.ref_delta_t_dt = self.get_epoch_of_first_observation(active_obs_only_for_ref_epoch)
-        # else:
-        #     self.ref_delta_t_dt = None
 
         # Loop over all surveys in the campaign:
         if verbose:
@@ -478,11 +473,13 @@ class Campaign:
         for survey_name, survey in self.surveys.items():
             if verbose:
                 print(f' - Survey: {survey_name}')
-            if survey.keep_survey:
+            if survey.is_active and survey.keep_survey:
                 survey.calculate_setup_data(obs_type=obs_type,
                                             ref_delta_t_campaign_dt=self.ref_delta_t_dt,
                                             active_obs_only_for_ref_epoch=active_obs_only_for_ref_epoch,
                                             verbose=verbose)
+            else:
+                survey.reset_setup_data(verbose)  # Remove setup data from previous calculations
 
     def get_epoch_of_first_observation(self, active_obs_only_for_ref_epoch=True):
         """Returns the epoch of the first (active) observation in this campaign.
@@ -580,7 +577,7 @@ class Campaign:
             raise ValueError('`ref_delta_t_dt` needs to be a datetime object.')
 
     def write_nsb_file(self, filename: str, lsm_run_index, vertical_offset_mode: str = 'first',
-                       exclude_datum_stations=False, verbose=False):
+                       exclude_datum_stations=False, formal_error_type='se', verbose=False):
         """Write the results of an LSM run to an nsb file (input for NSDB database).
 
         Notes
@@ -601,11 +598,19 @@ class Campaign:
             taken from the first setup at a station. (2) 'mean' indicates that mean values over all setups are taken.
         exclude_datum_stations : boolean, optional (default=False)
             `True` indicates that datum stations are excluded from the nsb file.
+        formal_error_type : str, optional (default=`se`, alternative: `sd`)
+            Select which formal errors (of station gravities) are exported to the nsd file. The user may choose
+            between post-fit standard deviations (`sd`) and standard errors ('se'). The standard errors consider the
+            noise floor defined in the estimation settings.
         verbose : bool, optional (default=False)
             If `True`, status messages are printed to the command line.
         """
         # Init.:
         nsb_string = ''
+        formal_error_type_options = ('sd', 'se')
+
+        if formal_error_type not in formal_error_type_options:
+            raise AssertionError(f'Invalid formal error type! Valid: "sd" and "se".')
 
         # Get and prepare data:
         # - lsm_run
@@ -676,11 +681,16 @@ class Campaign:
                     else:
                         raise AssertionError(f'Invalid choice for the nsb file comment: {WRITE_COMMENT_TO_NSB}!')
 
+                    if formal_error_type == 'se':
+                        formal_error = row['se_g_est_mugal']
+                    elif formal_error_type == 'sd':
+                        formal_error = row['sd_g_est_mugal']
+
                     nsb_string += '{:10s} {:8s}  {:9.0f} {:3.0f} {:1s}{:>5s} {:4.0f} {:4.0f}\n'.format(
                         station_name,
                         date_str,
                         row['g_est_mugal'] + ADDITIVE_CONST_ABS_GRTAVITY,
-                        row['sd_g_est_mugal'],
+                        formal_error,
                         GRAVIMETER_TYPES_KZG_LOOKUPTABLE[gravimeter_type],
                         comment_str,
                         (dhb_m + GRAVIMETER_REFERENCE_HEIGHT_CORRECTIONS_m[gravimeter_type]) * 100,
@@ -887,7 +897,7 @@ class Campaign:
                 tmp_filter = export_df['keep_obs']
             elif export_type == 'inactive_only':
                 tmp_filter = ~export_df['keep_obs']
-            export_obs_df = export_df.loc[tmp_filter, :]
+            export_df = export_df.loc[tmp_filter, :]
 
         # Export to CSV file:
         if verbose:
@@ -943,6 +953,7 @@ class Campaign:
                 if survey_name not in self.surveys:
                     flag_log_str += '  - Does not exist in this campaign.\n'
                 else:
+                    num_obs_in_survey = len(obs_list_df[obs_list_df['survey_name'] == survey_name])
                     for index, row in obs_list_df[obs_list_df['survey_name'] == survey_name].iterrows():
                         obs_epoch = row['obs_epoch']
                         if sys.version_info >= (3, 7):
@@ -963,8 +974,10 @@ class Campaign:
                             if self.surveys[survey_name].obs_df.loc[filter_tmp, 'keep_obs'].bool() != row['keep_obs']:
                                 self.surveys[survey_name].obs_df.loc[filter_tmp, 'keep_obs'] = row['keep_obs']
                                 count_changed += 1
-                    flag_log_str += f'  - Matched observations: {count_matched} of {len(obs_list_df)}\n'
+                    flag_log_str += f'  - Matched observations: {count_matched} of {num_obs_in_survey}\n'
                     flag_log_str += f'  - Changed "keep_obs" flags: {count_changed}\n'
+                    # Activate/deactivate survey according to the keep_obs flag of their observations:
+
         else:
             flag_log_str = 'Empty observation list file!'
         if verbose:
