@@ -22,6 +22,15 @@ import datetime as dt
 import pytz
 # from matplotlib import pyplot as plt
 
+# optional imports:
+try:
+    import geopandas
+except ImportError:
+    _has_geopandas = False
+else:
+    _has_geopandas = True
+    from shapely.geometry import Point, LineString  # Comes with geopandas
+
 from gravtools import settings
 from gravtools.models.lsm import LSM, create_hist, global_model_test, tau_test
 from gravtools.models import misc
@@ -44,23 +53,57 @@ class LSMDiff(LSM):
     # Column names of self.setup_obs_df:
     # - keys: Column names of the pandas dataframe
     # - values: Short description for table headers, etc., in the GUI
-    _SETUP_DIFF_COLUMNS_DICT = {
-        'survey_name': 'Survey',
-        'ref_epoch_dt': 'Epoch',
-        'diff_obs_id': 'Obs. ID',
-        'station_name_from': 'Station from',
-        'station_name_to': 'Station to',
-        'setup_id_from': 'Setup ID from',
-        'setup_id_to': 'Setup ID to',
-        'g_diff_mugal': 'g [µGal]',
-        'sd_g_diff_mugal': 'SD [µGal]',
-        'sd_g_diff_est_mugal': 'SD_est [µGal]',
-        'v_diff_mugal': 'Residuals [µGal]',  # Post fit residuals
-        'w_diff_mugal': 'Std. Residual []',
-        'r_diff_obs': 'Redundancy []',
-        'tau_test_result': 'Outlier Test',
+    # _SETUP_DIFF_COLUMNS_DICT = {
+    #     'survey_name': 'Survey',
+    #     'ref_epoch_dt': 'Epoch',
+    #     'diff_obs_id': 'Obs. ID',
+    #     'station_name_from': 'Station from',
+    #     'station_name_to': 'Station to',
+    #     'setup_id_from': 'Setup ID from',
+    #     'setup_id_to': 'Setup ID to',
+    #     'g_diff_mugal': 'g [µGal]',
+    #     'sd_g_diff_mugal': 'SD [µGal]',
+    #     'sd_g_diff_est_mugal': 'SD_est [µGal]',
+    #     'v_diff_mugal': 'Residuals [µGal]',  # Post fit residuals
+    #     'w_diff_mugal': 'Std. Residual []',
+    #     'r_diff_obs': 'Redundancy []',
+    #     'tau_test_result': 'Outlier Test',
+    # }
+    _SETUP_DIFF_COLUMNS_DTYPES = {
+        'survey_name': 'str',
+        'ref_epoch_dt': 'datetime64[ns, UTC]',
+        'diff_obs_id': 'int',
+        'station_name_from': 'str',
+        'station_name_to': 'str',
+        'setup_id_from': 'int',
+        'setup_id_to': 'int',
+        'g_diff_mugal': 'float',
+        'sd_g_diff_mugal': 'float',
+        'sd_g_diff_est_mugal': 'float',
+        'v_diff_mugal': 'float',  # Post fit residuals
+        'w_diff_mugal': 'float',
+        'r_diff_obs': 'float',
+        'tau_test_result': 'str',
+    }
+    _SETUP_DIFF_COLUMNS = list(_SETUP_DIFF_COLUMNS_DTYPES.keys())
+
+    # Short colum names with max. 10 char suitable for shapefiles
+    _SETUP_DIFF_COLUMNS_SHORT = {
+        'survey_name': 'survey',
+        'ref_epoch_dt': 'epoch_dt',  # dropped an
+        'diff_obs_id': 'obs_id',
+        'station_name_from': 'stat_from',
+        'station_name_to': 'stat_to',
+        'setup_id_from': 'setup_from',
+        'setup_id_to': 'setup_to',
+        'g_diff_mugal': 'dg',
+        'sd_g_diff_mugal': 'sd_dg',
+        'sd_g_diff_est_mugal': 'sd_dg_est',
+        'v_diff_mugal': 'v',  # Post fit residuals
+        'w_diff_mugal': 'w',
+        'r_diff_obs': 'r',
+        'tau_test_result': 'tau_test',
     }
-    _SETUP_DIFF_COLUMNS = list(_SETUP_DIFF_COLUMNS_DICT.keys())
 
     # Column names of self.drift_pol_df:
     # - keys: Column names of the pandas dataframe
@@ -136,6 +179,7 @@ class LSMDiff(LSM):
                confidence_level_chi_test=0.95,
                confidence_level_tau_test=0.95,
                drift_ref_epoch_type='survey',
+               noise_floor_mugal=0.0,
                verbose=False
                ):  # Confidence level):
         """Run the adjustment based on differential observations.
@@ -166,6 +210,10 @@ class LSMDiff(LSM):
             Defines whether the reference epoch t0 for the estimation of the drift polynomials for each survey in the
             campaign is the reference epoch of the first (active) observation in each survey (option: 'survey') or the
             first (active) observation in the whole campaign (option: 'campaign').
+        noise_floor_mugal : float, optional (default=0.0)
+            The standard error SE of the estimated gravity values at stations is calculated by SE = sqrt(SD**2 + NF**2),
+            where SD is the estimated standard deviation of the station's gravity and NF is the noise floor value
+            specified here.
         verbose : bool, optional (default=False)
             If True, status messages are printed to the command line, e.g. for debugging and testing
 
@@ -237,6 +285,7 @@ class LSMDiff(LSM):
             tmp_str += f'Scaling factor for datum constraints: {scaling_factor_datum_observations}\n'
             tmp_str += f'Scaling factor for SD of setup observations: {scaling_factor_for_sd_of_observations}\n'
             tmp_str += f'Additive const. to SD of setup obs. [µGal]: {add_const_to_sd_of_observations_mugal}\n'
+            tmp_str += f'Noise floor for std. error determination [µGal]: {noise_floor_mugal}\n'
             tmp_str += f'Confidence level Chi-test: {confidence_level_chi_test:4.2f}\n'
             tmp_str += f'Confidence level Tau-test: {confidence_level_tau_test:4.2f}\n'
             tmp_str += f'\n'
@@ -354,6 +403,7 @@ class LSMDiff(LSM):
                                                   None_list_placeholder,
                                                   )),
                                          columns=self._SETUP_DIFF_COLUMNS)
+        self.setup_obs_df = self.setup_obs_df.astype(self._SETUP_DIFF_COLUMNS_DTYPES)
 
         # - constraints:
         datum_station_id = -1
@@ -486,11 +536,16 @@ class LSMDiff(LSM):
         tau_test_result, tau_critical_value = tau_test(mat_w=mat_w, dof=dof, alpha=alpha_tau, mat_r=mat_r)
         number_of_outliers = tau_test_result.count("failed")
 
+        tau_test_result_diff_obs = tau_test_result[:number_of_diff_obs]
+        tau_test_result_pseudo_obs = tau_test_result[number_of_diff_obs:]
+
         if verbose or self.write_log:
             tmp_str = f'\n'
             tmp_str += f'# Tau-test results:\n'
             tmp_str += f'Critical value: {tau_critical_value:1.3f}\n'
-            tmp_str += f' - Number of detected outliers: {number_of_outliers}\n'
+            tmp_str += f' - Number of detected outliers (total): {number_of_outliers}\n'
+            tmp_str += f' - Number of detected outliers (observations): {tau_test_result_diff_obs.count("failed")}\n'
+            tmp_str += f' - Number of detected outliers (datum constraints): {tau_test_result_pseudo_obs.count("failed")}\n'
             tmp_str += f' - Number low redundancy component: {tau_test_result.count("r too small")}\n'
             tmp_str += f'\n'
             if verbose:
@@ -515,18 +570,16 @@ class LSMDiff(LSM):
         w_pseudo_obs_mugal = mat_w[number_of_diff_obs:]
         r_diff_obs = mat_r[:number_of_diff_obs]
         r_pseudo_obs = mat_r[number_of_diff_obs:]
-        # TODO: Add Tau criterion data here
-        tau_test_result_diff_obs = tau_test_result[:number_of_diff_obs]
-        tau_test_result_pseudo_obs = tau_test_result[number_of_diff_obs:]
 
         # Station related results:
         for idx, stat_name in enumerate(self.observed_stations):
             filter_tmp = self.stat_obs_df['station_name'] == stat_name
             self.stat_obs_df.loc[filter_tmp, 'g_est_mugal'] = g_est_mugal[idx]
             self.stat_obs_df.loc[filter_tmp, 'sd_g_est_mugal'] = sd_g_est_mugal[idx]
+            self.stat_obs_df.loc[filter_tmp, 'se_g_est_mugal'] = np.sqrt(sd_g_est_mugal[idx]**2 + noise_floor_mugal**2)
         # Calculate differences to estimates:
         self.stat_obs_df['diff_g_est_mugal'] = self.stat_obs_df['g_est_mugal'] - self.stat_obs_df['g_mugal']
-        self.stat_obs_df['diff_sd_g_est_mugal'] = self.stat_obs_df['sd_g_est_mugal'] - self.stat_obs_df['sd_g_mugal']
+        self.stat_obs_df['diff_se_g_est_mugal'] = self.stat_obs_df['se_g_est_mugal'] - self.stat_obs_df['sd_g_mugal']
 
         # Drift parameters:
         survey_name_list = []
@@ -575,7 +628,7 @@ class LSMDiff(LSM):
             tmp_str += f' - Station data:\n'
             tmp_str += self.stat_obs_df[['station_name', 'is_datum', 'g_mugal', 'g_est_mugal',
                                          'diff_g_est_mugal', 'sd_g_mugal',
-                                         'sd_g_est_mugal']].to_string(index=False,
+                                         'sd_g_est_mugal', 'se_g_est_mugal']].to_string(index=False,
                                                                       float_format=lambda x: '{:.1f}'.format(x))
             tmp_str += f'\n\n'
             tmp_str += f' - Drift polynomial coefficients:\n'
@@ -617,3 +670,65 @@ class LSMDiff(LSM):
         self.global_model_test_status = chi_test
         self.number_of_outliers = number_of_outliers
         self.drift_ref_epoch_type = drift_ref_epoch_type
+
+    def export_obs_results_shapefile(self, filename, epsg_code, verbose=True):
+        """Export observation-related results from the `setup_obs_df` dataframe to a shapefile.
+
+        Notes
+        -----
+        This method relies on the optional module `geopandas` (optional dependency).
+
+        Parameters
+        ---------
+        filename : str
+            Name and path of the output shapefile.
+        epsg_code: int
+            EPSG code of the station coordinates' CRS.
+        verbose : bool, optional (default=False)
+            `True` implies that status messages are printed to the command line.
+        """
+        if not _has_geopandas:
+            raise ImportError(f'Optional dependency "geopandas" not available, but needed for writing shapefiles!')
+        if verbose:
+            print(f'Save observation results of lsm run "{self.comment}" to: {filename}')
+
+        setup_obs_df = self.setup_obs_df.copy(deep=True)
+
+        # Change dtypes (just to be save!):
+        setup_obs_df = setup_obs_df.astype(self._SETUP_DIFF_COLUMNS_DTYPES)
+
+        setup_obs_df['comment'] = self.comment
+
+        # Get coordinates:
+        stat_obs_df_short = self.stat_obs_df[['station_name', 'long_deg', 'lat_deg']].copy(deep=True)
+        setup_obs_df = setup_obs_df.merge(stat_obs_df_short, left_on='station_name_from', right_on='station_name',
+                                          how='left')
+        setup_obs_df.rename(columns={'long_deg': 'from_long', 'lat_deg': 'from_lat'}, inplace=True)
+        setup_obs_df.drop(columns=['station_name'], inplace=True)
+
+        setup_obs_df = setup_obs_df.merge(stat_obs_df_short, left_on='station_name_to', right_on='station_name',
+                                          how='left')
+        setup_obs_df.rename(columns={'long_deg': 'to_long', 'lat_deg': 'to_lat'}, inplace=True)
+        setup_obs_df.drop(columns=['station_name'], inplace=True)
+
+        # Further changes:
+        setup_obs_df['ref_epoch'] = setup_obs_df['ref_epoch_dt'].dt.strftime('%Y-%m-%d %H:%M:%S')
+
+        # Rename columns to short names with max 10 chars suitable for shapefiles:
+        setup_obs_df.rename(columns=self._SETUP_DIFF_COLUMNS_SHORT, inplace=True)
+        setup_obs_df.drop(columns=['epoch_dt'], inplace=True)  # datetime fields cannot be converted to shapefiles!
+
+        # Point => LineString
+        # https://stackoverflow.com/questions/55070635/single-row-points-to-linestring
+        # -
+        from_points = [Point(xy) for xy in zip(setup_obs_df['from_long'], setup_obs_df['from_lat'])]
+        to_points = [Point(xy) for xy in zip(setup_obs_df['to_long'], setup_obs_df['to_lat'])]
+
+        lines = [LineString(xy) for xy in zip(from_points, to_points)]
+
+        # Create GeoDataFrame:
+        setup_obs_gdf = geopandas.GeoDataFrame(setup_obs_df)
+        setup_obs_gdf.set_geometry(lines, crs=epsg_code, inplace=True)
+
+        # Save to shapefile:
+        setup_obs_gdf.to_file(filename)
