@@ -26,6 +26,14 @@ from gravtools.models.lsm import LSM, create_hist, global_model_test, tau_test
 from gravtools.models import misc
 from gravtools import __version__ as GRAVTOOLS_VERSION
 
+# optional imports:
+try:
+    import geopandas
+except ImportError:
+    _has_geopandas = False
+else:
+    _has_geopandas = True
+
 
 class LSMNonDiff(LSM):
     """Least-squares adjustment of non-differential gravimeter observations with weighted constraints.
@@ -43,21 +51,51 @@ class LSMNonDiff(LSM):
     # Column names of self.setup_obs_df:
     # - keys: Column names of the pandas dataframe
     # - values: Short description for table headers, etc., in the GUI
-    _SETUP_OBS_COLUMNS_DICT = {
-        'survey_name': 'Survey',
-        'ref_epoch_dt': 'Epoch',
-        'obs_id': 'Obs. ID',
-        'station_name': 'Station',
-        'setup_id': 'Setup ID',
-        'g_obs_mugal': 'g [µGal]',
-        'sd_g_obs_mugal': 'SD [µGal]',
-        'sd_g_obs_est_mugal': 'SD_est [µGal]',
-        'v_obs_est_mugal': 'Residuals [µGal]',  # Post fit residuals
-        'w_obs_est_mugal': 'Std. Residual []',
-        'r_obs_est': 'Redundancy []',
-        'tau_test_result': 'Outlier Test',
+    #_SETUP_OBS_COLUMNS_DICT = {
+    #    'survey_name': 'Survey',
+    #    'ref_epoch_dt': 'Epoch',
+    #    'obs_id': 'Obs. ID',
+    #    'station_name': 'Station',
+    #    'setup_id': 'Setup ID',
+    #    'g_obs_mugal': 'g [µGal]',
+    #    'sd_g_obs_mugal': 'SD [µGal]',
+    #    'sd_g_obs_est_mugal': 'SD_est [µGal]',
+    #    'v_obs_est_mugal': 'Residuals [µGal]',  # Post fit residuals
+    #    'w_obs_est_mugal': 'Std. Residual []',
+    #    'r_obs_est': 'Redundancy []',
+    #    'tau_test_result': 'Outlier Test',
+    #}
+    _SETUP_OBS_COLUMNS_DTYPES = {
+        'survey_name': 'str',
+        'ref_epoch_dt': 'datetime64[ns, UTC]',
+        'obs_id': 'int',
+        'station_name': 'str',
+        'setup_id': 'int',
+        'g_obs_mugal': 'float',
+        'sd_g_obs_mugal': 'float',
+        'sd_g_obs_est_mugal': 'float',
+        'v_obs_est_mugal': 'float',  # Post fit residuals
+        'w_obs_est_mugal': 'float',
+        'r_obs_est': 'float',
+        'tau_test_result': 'str',
+    }
+    _SETUP_OBS_COLUMNS = list(_SETUP_OBS_COLUMNS_DTYPES.keys())
+
+    # Short colum names with max. 10 char suitable for shapefiles:
+    _SETUP_OBS_COLUMNS_SHORT = {
+        'survey_name': 'survey',
+        'ref_epoch_dt': 'epoch_dt',
+        'obs_id': 'obs_id',
+        'station_name': 'station',
+        'setup_id': 'setup',
+        'g_obs_mugal': 'g',
+        'sd_g_obs_mugal': 'sd_g',
+        'sd_g_obs_est_mugal': 'sd_g_est',
+        'v_obs_est_mugal': 'v',  # Post fit residuals
+        'w_obs_est_mugal': 'w',
+        'r_obs_est': 'r',
+        'tau_test_result': 'tau_test',
     }
-    _SETUP_OBS_COLUMNS = list(_SETUP_OBS_COLUMNS_DICT.keys())
 
     # Column names of self.drift_pol_df:
     # - keys: Column names of the pandas dataframe
@@ -123,7 +161,7 @@ class LSMNonDiff(LSM):
         :py:obj:`.LSMNonDiff`
             Contains all information required for adjusting the campaign.
         """
-        return super().from_campaign(campaign, comment='', write_log=True)
+        return super().from_campaign(campaign, comment=comment, write_log=True)
 
     def adjust(self, drift_pol_degree=1,
                sig0_mugal=1,
@@ -133,6 +171,7 @@ class LSMNonDiff(LSM):
                confidence_level_chi_test=0.95,
                confidence_level_tau_test=0.95,
                drift_ref_epoch_type='survey',
+               noise_floor_mugal=0.0,
                verbose=False
                ):  # Confidence level):
         """Run the adjustment based on non-differential observations.
@@ -163,6 +202,10 @@ class LSMNonDiff(LSM):
             Defines whether the reference epoch t0 for the estimation of the drift polynomials for each survey in the
             campaign is the reference epoch of the first (active) observation in each survey (option: 'survey') or the
             first (active) observation in the whole campaign (option: 'campaign').
+        noise_floor_mugal : float, optional (default=0.0)
+            The standard error SE of the estimated gravity values at stations is calculated by SE = sqrt(SD**2 + NF**2),
+            where SD is the estimated standard deviation of the station's gravity and NF is the noise floor value
+            specified here.
         verbose : bool, optional (default=False)
             If `True`, status messages are printed to the command line, e.g. for debugging and testing
         """
@@ -229,6 +272,7 @@ class LSMNonDiff(LSM):
             tmp_str += f'Scaling factor for datum constraints: {scaling_factor_datum_observations}\n'
             tmp_str += f'Scaling factor for SD of setup observations: {scaling_factor_for_sd_of_observations}\n'
             tmp_str += f'Additive const. to SD of setup obs. [µGal]: {add_const_to_sd_of_observations_mugal}\n'
+            tmp_str += f'Noise floor for std. error determination [µGal]: {noise_floor_mugal}\n'
             tmp_str += f'Confidence level Chi-test: {confidence_level_chi_test:4.2f}\n'
             tmp_str += f'Confidence level Tau-test: {confidence_level_tau_test:4.2f}\n'
             tmp_str += f'\n'
@@ -324,6 +368,7 @@ class LSMNonDiff(LSM):
                                                   None_list_placeholder,
                                                   )),
                                          columns=self._SETUP_OBS_COLUMNS)
+        self.setup_obs_df = self.setup_obs_df.astype(self._SETUP_OBS_COLUMNS_DTYPES)
 
         # - constraints:
         datum_station_id = -1
@@ -437,7 +482,7 @@ class LSMNonDiff(LSM):
                 self.log_str += tmp_str
 
         # outlier detection effectiveness (redundancy components)
-        diag_Qvv = np.diag(mat_Qvv)  # TODO: Still needed?
+        # diag_Qvv = np.diag(mat_Qvv)
         # mat_R = np.diag(mat_P) * diag_Qvv
         # mat_R is exactly the same as "mat_r"
 
@@ -455,11 +500,16 @@ class LSMNonDiff(LSM):
         tau_test_result, tau_critical_value = tau_test(mat_w=mat_w, dof=dof, alpha=alpha_tau, mat_r=mat_r)
         number_of_outliers = tau_test_result.count("failed")
 
+        tau_test_result_obs = tau_test_result[:number_of_observations]
+        tau_test_result_pseudo_obs = tau_test_result[number_of_observations:]
+
         if verbose or self.write_log:
             tmp_str = f'\n'
             tmp_str += f'# Tau-test results:\n'
             tmp_str += f'Critical value: {tau_critical_value:1.3f}\n'
             tmp_str += f' - Number of detected outliers: {number_of_outliers}\n'
+            tmp_str += f' - Number of detected outliers (observations): {tau_test_result_obs.count("failed")}\n'
+            tmp_str += f' - Number of detected outliers (datum constraints): {tau_test_result_pseudo_obs.count("failed")}\n'
             tmp_str += f' - Number low redundancy component: {tau_test_result.count("r too small")}\n'
             tmp_str += f'\n'
             if verbose:
@@ -484,18 +534,17 @@ class LSMNonDiff(LSM):
         w_pseudo_obs_mugal = mat_w[number_of_observations:]
         r_obs_est = mat_r[:number_of_observations]
         r_pseudo_obs = mat_r[number_of_observations:]
-        # TODO: Add Tau criterion data here
-        tau_test_result_obs = tau_test_result[:number_of_observations]
-        tau_test_result_pseudo_obs = tau_test_result[number_of_observations:]
 
         # Station related results:
         for idx, stat_name in enumerate(self.observed_stations):
             filter_tmp = self.stat_obs_df['station_name'] == stat_name
             self.stat_obs_df.loc[filter_tmp, 'g_est_mugal'] = g_est_mugal[idx]
             self.stat_obs_df.loc[filter_tmp, 'sd_g_est_mugal'] = sd_g_est_mugal[idx]
+            self.stat_obs_df.loc[filter_tmp, 'se_g_est_mugal'] = np.sqrt(
+                sd_g_est_mugal[idx] ** 2 + noise_floor_mugal ** 2)
         # Calculate differences to estimates:
         self.stat_obs_df['diff_g_est_mugal'] = self.stat_obs_df['g_est_mugal'] - self.stat_obs_df['g_mugal']
-        self.stat_obs_df['diff_sd_g_est_mugal'] = self.stat_obs_df['sd_g_est_mugal'] - self.stat_obs_df['sd_g_mugal']
+        self.stat_obs_df['diff_se_g_est_mugal'] = self.stat_obs_df['se_g_est_mugal'] - self.stat_obs_df['sd_g_mugal']
 
         # Drift parameters:
         survey_name_list = []
@@ -546,7 +595,7 @@ class LSMNonDiff(LSM):
             tmp_str += f' - Station data:\n'
             tmp_str += self.stat_obs_df[['station_name', 'is_datum', 'g_mugal', 'g_est_mugal',
                                                 'diff_g_est_mugal', 'sd_g_mugal',
-                                                'sd_g_est_mugal']].to_string(index=False,
+                                                'sd_g_est_mugal', 'se_g_est_mugal']].to_string(index=False,
                                                                              float_format=lambda x: '{:.1f}'.format(x))
             tmp_str += f'\n\n'
             tmp_str += f' - Drift polynomial coefficients:\n'
@@ -585,4 +634,44 @@ class LSMNonDiff(LSM):
         self.x_estimate_names = self.observed_stations + x_estimate_drift_coeff_names
         self.global_model_test_status = chi_test
         self.number_of_outliers = number_of_outliers
-        self.drift_ref_epoch_type = drift_ref_epoch_type
\ No newline at end of file
+        self.drift_ref_epoch_type = drift_ref_epoch_type
+
+    def export_obs_results_shapefile(self, filename, epsg_code, verbose=True):
+        """Export observation-related results from the `setup_obs_df` dataframe to a shapefile.
+
+        Notes
+        -----
+        This method relies on the optional module `geopandas` (optional dependency).
+
+        Parameters
+        ---------
+        filename : str
+            Name and path of the output shapefile.
+        epsg_code: int
+            EPSG code of the station coordinates' CRS.
+        verbose : bool, optional (default=False)
+            `True` implies that status messages are printed to the command line.
+        """
+        if not _has_geopandas:
+            raise ImportError(f'Optional dependency "geopandas" not available, but needed for writing shapefiles!')
+        if verbose:
+            print(f'Save observation results of lsm run "{self.comment}" to: {filename}')
+        setup_obs_df = self.setup_obs_df.copy(deep=True)
+        # Change dtypes (just to be save!):
+        setup_obs_df = setup_obs_df.astype(self._SETUP_OBS_COLUMNS_DTYPES)
+        setup_obs_df['comment'] = self.comment
+
+        # Get coordinates:
+        stat_obs_df_short = self.stat_obs_df[['station_name', 'long_deg', 'lat_deg']].copy(deep=True)
+        setup_obs_df = setup_obs_df.merge(stat_obs_df_short, left_on='station_name', right_on='station_name',
+                                          how='left')
+
+        # Rename columns to short names with max 10 chars suitable for shapefiles:
+        setup_obs_df.rename(columns=self._SETUP_OBS_COLUMNS_SHORT, inplace=True)
+        setup_obs_df.drop(columns=['epoch_dt'], inplace=True)  # datetime fields cannot be converted to shapefiles!
+
+        setup_obs_gdf = geopandas.GeoDataFrame(setup_obs_df,
+                                              geometry=geopandas.points_from_xy(setup_obs_df['long_deg'],
+                                                                                setup_obs_df['lat_deg']),
+                                              crs=epsg_code)
+        setup_obs_gdf.to_file(filename)
