@@ -21,14 +21,14 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import pandas as pd
 import numpy as np
-from scipy import stats
+# from scipy import stats
 import datetime as dt
 import pytz
-import copy
+# import copy
 from sklearn.linear_model import LinearRegression
 
 from gravtools.models.lsm import LSM
-from gravtools import settings
+# from gravtools import settings
 from gravtools import __version__ as GRAVTOOLS_VERSION
 
 
@@ -319,8 +319,8 @@ class BEVLegacyProcessing(LSM):
         # self.stat_obs_df['g_est_full_mugal'] = self.stat_obs_df['g_est_mugal'] + 9.8e8  # Get absolute gravity value at station [µGal]
 
         # Add g0 and sig_go to "self.stat_obs_df":
-        self.stat_obs_df['g0_mugal'] = None
-        self.stat_obs_df['sig_g0_mugal'] = None
+        self.stat_obs_df['g0_mugal'] = np.NAN
+        self.stat_obs_df['sig_g0_mugal'] = np.NAN
         self.stat_obs_df.update(stat_df_datum[['g0_mugal', 'sig_g0_mugal']])
 
         # Write log:
