@@ -66,6 +66,23 @@ class Station:
             String that lists the names of the surveys in which the station was observed in (separated by ';')
     """
 
+    _STAT_DF_COLUMNS_DTYPES = {
+        'station_name': 'str',  # Station name, str
+        'long_deg': 'float',  # longitude [deg], float
+        'lat_deg': 'float',  # latitude [deg], float
+        'height_m': 'float',  # Height [m]
+        'g_mugal': 'float',  # gravity [µGal]
+        'sd_g_mugal': 'float',  # standard deviation of the gravity [µGal]
+        'vg_mugalm': 'float',  # vertical gradient [µGal/m]
+        'is_observed': 'bool',  # flag: True, if station was observed in at least one survey in the current campaign.
+        'source_type': 'str',  # Source of the station data, str
+        'source_name': 'str',  # Name of the data source
+        'is_datum': 'bool',  # flag, that indicates whether this is a datum station, bool
+        'in_survey': 'str',  # Names of surveys in which ths station was observed (separator = ;), str
+    }
+    _STAT_DF_COLUMNS = list(_STAT_DF_COLUMNS_DTYPES.keys())
+
+
     _STAT_DF_COLUMNS = (
         'station_name',  # Station name, str
         'long_deg',  # longitude [deg], float
@@ -106,6 +123,7 @@ class Station:
 
         # Init. station dataframe:
         self.stat_df = pd.DataFrame(columns=self._STAT_DF_COLUMNS)
+        self.stat_df = self.stat_df.astype(self._STAT_DF_COLUMNS_DTYPES)
 
         # Loop over station_files and append the content to stat_df:
         for file_name, file_type in station_files.items():
@@ -168,7 +186,14 @@ class Station:
             if col_name not in self._STAT_DF_COLUMNS:
                 columns_to_be_dropped.append(col_name)
 
-        stat_df_oesgn = pd.read_fwf(filename, widths=widths, header=None)
+        try:
+            stat_df_oesgn = pd.read_fwf(filename, widths=widths, header=None)
+        except UnicodeDecodeError:
+            # To catch encoding errors with files created/converted on Windows.
+            # See: https://docs.python.org/3/library/codecs.html#standard-encodings
+            # See: https://stackoverflow.com/questions/45529507/unicodedecodeerror-utf-8-codec-cant-decode-byte-0x96-in-position-35-invalid
+            stat_df_oesgn = pd.read_fwf(filename, widths=widths, header=None, encoding='ISO-8859-1')
+
         stat_df_oesgn.columns = column_names
         stat_df_oesgn['is_observed'] = False  # Default = False
         if is_datum:
@@ -324,6 +349,7 @@ class Station:
         tmp_df['source_type'] = 'obs_file'
         tmp_df['source_name'] = survey.name  # survey name
         tmp_df.reset_index(drop=True, inplace=True)
+        tmp_df = tmp_df.astype(self._STAT_DF_COLUMNS_DTYPES)
 
         self.add_stations(tmp_df, data_source_type='obs_file', verbose=verbose)
 
